@@ -285,40 +285,49 @@ ParsedFilePointer Driver::translationUnit(const QString& fileName) const
 class Driver::ParseHelper
 {
 public:
-    ParseHelper(const QString& fileName, bool force, Driver* driver, bool reportMessages = true, QString includedFrom = QString()) : m_wasReset(false), m_fileName(fileName), m_previousFileName(driver->m_currentFileName),  m_previousLexer(driver->lexer), m_previousParsedFile(driver->m_currentParsedFile), m_previousCachedLexedFile(driver->m_currentLexerCache), m_force(force), m_driver(driver), m_lex(m_driver)
+    ParseHelper(const QString& fileName, bool force, Driver* driver, bool reportMessages = true, QString includedFrom = QString())
+    : m_wasReset(false), m_fileName(fileName), m_previousFileName(driver->m_currentFileName),  m_previousLexer(driver->lexer),
+      m_previousParsedFile(driver->m_currentParsedFile), m_previousCachedLexedFile(driver->m_currentLexerCache), m_force(force),
+      m_reportMessages(reportMessages), m_includedFrom(includedFrom), m_driver(driver), m_lex(m_driver) {}
+
+    /**
+     * This function must be called after constructing the ParseHelper.
+     */
+    void init()
     {
-        QFileInfo fileInfo(fileName);
-        m_driver->m_currentParsedFile = new ParsedFile(fileName, fileInfo.lastModified());
-        if (!includedFrom.isEmpty())
-            m_driver->m_currentParsedFile->setIncludedFrom(includedFrom);
+        QFileInfo fileInfo(m_fileName);
+        m_driver->m_currentParsedFile = new ParsedFile(m_fileName, fileInfo.lastModified());
+        if (!m_includedFrom.isEmpty())
+            m_driver->m_currentParsedFile->setIncludedFrom(m_includedFrom);
 #ifdef CACHELEXER
-        m_driver->m_currentLexerCache = new CachedLexedFile(fileName, &m_driver->m_lexerCache);
+        m_driver->m_currentLexerCache = new CachedLexedFile(m_fileName, &m_driver->m_lexerCache);
 #endif
         m_absoluteFilePath = fileInfo.absoluteFilePath();
 
         QMap<QString, ParsedFilePointer>::Iterator it = m_driver->m_parsedUnits.find(m_absoluteFilePath);
 
-        if (force && it != m_driver->m_parsedUnits.end()) {
+        if (m_force && it != m_driver->m_parsedUnits.end()) {
             m_driver->takeTranslationUnit(m_absoluteFilePath);
         } else if (it != m_driver->m_parsedUnits.end() && *it != (ParsedFilePointer)0) {
             // file already processed
             return ;
         }
 
-        CachedLexedFilePointer lexedFileP = m_driver->m_lexerCache.lexedFile(HashedString(fileName));
+        CachedLexedFilePointer lexedFileP = m_driver->m_lexerCache.lexedFile(HashedString(m_fileName));
 
-        m_driver->m_dependences.remove(fileName);
-        m_driver->m_problems.remove(fileName);
+        m_driver->m_dependences.remove(m_fileName);
+        m_driver->m_problems.remove(m_fileName);
 
-        driver->m_currentFileName = fileName;
+        m_driver->m_currentFileName = m_fileName;
 
         m_driver->lexer = &m_lex;
         m_driver->setupLexer(&m_lex);
 
-        m_lex.setReportMessages(reportMessages);
+        m_lex.setReportMessages(m_reportMessages);
 
-        DBG_DRV << "lexing file " << fileName << endl;
-        m_lex.setSource(m_driver->sourceProvider() ->contents(fileName));
+        DBG_DRV << "lexing file " << m_fileName << endl;
+        m_fileContent = m_driver->sourceProvider()->contents(m_fileName);
+        m_lex.setSource(m_fileContent);
         if (m_previousCachedLexedFile)
             m_previousCachedLexedFile->merge(*m_driver->m_currentLexerCache);
         else
@@ -385,8 +394,11 @@ private:
     ParsedFilePointer m_previousParsedFile;
     CachedLexedFilePointer m_previousCachedLexedFile;
     bool m_force;
+    bool m_reportMessages;
+    QString m_includedFrom;
     Driver* m_driver;
     Lexer m_lex;
+    QString m_fileContent;
 };
 
 
@@ -441,7 +453,7 @@ void Driver::addDependence(const QString & fileName, const Dependence & dep)
     CachedLexedFilePointer lexedFileP = m_lexerCache.lexedFile(HashedString(file));
     if (lexedFileP) {
         CachedLexedFile& lexedFile(*lexedFileP);
-        m_currentLexerCache->merge(lexedFile); //The ParseHelper will will copy the include-files into the result later
+        m_currentLexerCache->merge(lexedFile); //The ParseHelper will copy the include-files into the result later
         for (MacroSet::Macros::const_iterator it = lexedFile.definedMacros().macros().begin(); it != lexedFile.definedMacros().macros().end(); ++it) {
             addMacro((*it));
         }
@@ -450,6 +462,7 @@ void Driver::addDependence(const QString & fileName, const Dependence & dep)
     }
 
     ParseHelper h(file, true, this, false, m_currentMasterFileName);
+    h.init();
 
     /*if (m_parsedUnits.find(file) != m_parsedUnits.end())
         return;*/
@@ -553,6 +566,7 @@ bool Driver::parseFile(const QString& fileName, bool onlyPreProcess, bool force
     m_includePaths = getCustomIncludePath(fileName);
 
     ParseHelper p(fileName, force, this);
+    p.init();
     if (!onlyPreProcess) {
         p.parse();
     }
@@ -789,6 +803,9 @@ void Driver::setupLexer(Lexer * lexer)
     if (!hasMacro("__const__")) addMacro(Macro(QLatin1String("__const__"), QLatin1String("const")));
     if (!hasMacro("__volatile__")) addMacro(Macro(QLatin1String("__volatile__"), QLatin1String("volatile")));
     if (!hasMacro("__complex__")) addMacro(Macro(QLatin1String("__complex__"), QString()));
+
+    // c++ macros
+    if (!hasMacro("__cplusplus")) addMacro(Macro(QLatin1String("__cplusplus"), QString()));
 }
 
 void Driver::setupParser(Parser * parser)
