@@ -21,6 +21,7 @@
 
 class CodeBlockWithComments;
 class CodeComment;
+class QXmlStreamWriter;
 class TextBlock;
 class UMLPackage;
 
@@ -64,7 +65,7 @@ public:
 
     virtual QString toString ();
 
-    virtual void saveToXMI1 (QDomDocument & doc, QDomElement & root);
+    virtual void saveToXMI1(QXmlStreamWriter& writer);
     virtual void loadFromXMI1 (QDomElement & root);
 
     virtual CodeBlock * newCodeBlock ();
@@ -83,7 +84,7 @@ public:
 
 protected:
 
-    virtual void setAttributesOnNode (QDomDocument & doc, QDomElement & blockElement);
+    virtual void setAttributesOnNode (QXmlStreamWriter& writer);
 
     virtual void setAttributesFromNode (QDomElement & element);
 
