@@ -5,7 +5,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   copyright (C) 2003      Brian Thomas <thomas@mail630.gsfc.nasa.gov>   *
- *   copyright (C) 2004-2014                                               *
+ *   copyright (C) 2004-2020                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -25,24 +25,24 @@ CPPCodeComment::~CPPCodeComment ()
 {
 }
 
-void CPPCodeComment::saveToXMI (QDomDocument & doc, QDomElement & root)
+void CPPCodeComment::saveToXMI1(QXmlStreamWriter& writer)
 {
-    QDomElement blockElement = doc.createElement("cppcodecomment");
-    setAttributesOnNode(doc, blockElement); // as we added no additional fields to this class we may
+    writer.writeStartElement(QLatin1String("cppcodecomment"));
+    setAttributesOnNode(writer); // as we added no additional fields to this class we may
     // just use parent TextBlock method
-    root.appendChild(blockElement);
+    writer.writeEndElement();
 }
 
 QString CPPCodeComment::toString () const
 {
-    QString output = "";
+    QString output;
 
     // simple output method
     if(getWriteOutText())
     {
         QString indent = getIndentationString();
         QString endLine = getNewLineEndingChars();
-        output.append(formatMultiLineText (getText()+endLine, indent +"// ", endLine));
+        output.append(formatMultiLineText (getText() + endLine, indent + QLatin1String("// "), endLine));
     }
 
     return output;
@@ -50,7 +50,7 @@ QString CPPCodeComment::toString () const
 
 QString CPPCodeComment::getNewEditorLine (int amount)
 {
-    QString line = getIndentationString(amount) + "// ";
+    QString line = getIndentationString(amount) + QLatin1String("// ");
     return line;
 }
 
@@ -60,6 +60,6 @@ QString CPPCodeComment::unformatText (const QString & text, const QString & inde
     QString mytext = TextBlock::unformatText(text, indent);
 
     // now leading slashes
-    mytext.remove(QRegExp("^\\/\\/\\s*"));
+    mytext.remove(QRegExp(QLatin1String("^\\/\\/\\s*")));
     return mytext;
 }
