@@ -5,7 +5,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   copyright (C) 2003      Brian Thomas <thomas@mail630.gsfc.nasa.gov>   *
- *   copyright (C) 2004-2014                                               *
+ *   copyright (C) 2004-2020                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -21,6 +21,9 @@
 #include "debug_utils.h"
 #include "uml.h"
 
+// qt/kde includes
+#include <QXmlStreamWriter>
+
 /**
  * Constructor
  */
@@ -237,29 +240,29 @@ void HierarchicalCodeBlock::addCodeClassFieldMethods(CodeClassFieldList &list)
 /**
  * Save the XMI representation of this object
  */
-void HierarchicalCodeBlock::saveToXMI1 (QDomDocument & doc, QDomElement & root)
+void HierarchicalCodeBlock::saveToXMI1(QXmlStreamWriter& writer)
 {
-    QDomElement blockElement = doc.createElement(QLatin1String("hierarchicalcodeblock"));
-    setAttributesOnNode(doc, blockElement);
-    root.appendChild(blockElement);
+    writer.writeStartElement(QLatin1String("hierarchicalcodeblock"));
+    setAttributesOnNode(writer);
+    writer.writeEndElement();
 }
 
 /**
  * set attributes of the node that represents this class
  * in the XMI document.
  */
-void HierarchicalCodeBlock::setAttributesOnNode (QDomDocument & doc, QDomElement & elem)
+void HierarchicalCodeBlock::setAttributesOnNode (QXmlStreamWriter& writer)
 {
     // set super-class attributes
-    CodeBlockWithComments::setAttributesOnNode(doc, elem);
-    CodeGenObjectWithTextBlocks::setAttributesOnNode(doc, elem);
+    CodeBlockWithComments::setAttributesOnNode(writer);
+    CodeGenObjectWithTextBlocks::setAttributesOnNode(writer);
 
     // set local class attributes
     if (contentType() != CodeBlock::AutoGenerated)
     {
         QString endLine = UMLApp::app()->commonPolicy()->getNewLineEndingChars();
-        elem.setAttribute(QLatin1String("startText"), encodeText(getStartText(), endLine));
-        elem.setAttribute(QLatin1String("endText"), encodeText(getEndText(), endLine));
+        writer.writeAttribute(QLatin1String("startText"), encodeText(getStartText(), endLine));
+        writer.writeAttribute(QLatin1String("endText"), encodeText(getEndText(), endLine));
     }
 }
 
