@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2003-2014                                               *
+ *   copyright (C) 2003-2020                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -22,7 +22,7 @@
  * @short Displays a box.
  * @author Jonathan Riddell
  * @see UMLWidget
- * Bugs and comments to umbrello-devel@kde.org or http://bugs.kde.org
+ * Bugs and comments to umbrello-devel@kde.org or https://bugs.kde.org
  */
 class BoxWidget : public UMLWidget
 {
@@ -31,9 +31,9 @@ public:
     virtual ~BoxWidget();
 
     virtual void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0);
-    virtual void saveToXMI1(QDomDocument& qDoc, QDomElement& qElement);
+    virtual void saveToXMI1(QXmlStreamWriter& writer);
 
-    virtual void showPropertiesDialog();
+    virtual bool showPropertiesDialog();
 
 protected:
     void toForeground();
